@@ -1,8 +1,22 @@
 ## Wrapper function for gowerD
 gowerD <- function(data.x, data.y = data.x, weights=NULL,numerical,factors,orders,mixed,levOrders,mixed.constant) {
   maxplus1 <- function(x){
-    x[is.na(x)] <- max(x,na.rm=TRUE)+1
-    x
+    if(all(is.na(x)))
+      return(1)
+    else
+      return(max(x,na.rm=TRUE)+1)
+  }
+  min0 <- function(x,na.rm){
+    if(all(is.na(x)))
+      return(0)
+    else
+      return(min(x,na.rm=na.rm))
+  }
+  max1 <- function(x,na.rm){
+    if(all(is.na(x)))
+      return(1)
+    else
+      return(max(x,na.rm=na.rm))
   }
   #weights <- rep(1,ncol(data.x))
   for(i in 1:ncol(data.x)){
@@ -14,8 +28,8 @@ gowerD <- function(data.x, data.y = data.x, weights=NULL,numerical,factors,order
   data.y <- data.y[,c(numerical,factors,orders,mixed),drop=FALSE]
   if(length(numerical)>0){
     ##Datensatz durch Range dividieren
-    rmin <- apply(rbind(apply(data.x[,numerical,drop=FALSE],2,min,na.rm=TRUE),apply(data.y[,numerical,drop=FALSE],2,min,na.rm=TRUE)),2,min,na.rm=TRUE)
-    rmax <- apply(rbind(apply(data.x[,numerical,drop=FALSE],2,max,na.rm=TRUE),apply(data.y[,numerical,drop=FALSE],2,max,na.rm=TRUE)),2,max,na.rm=TRUE)
+    rmin <- apply(rbind(apply(data.x[,numerical,drop=FALSE],2,min,na.rm=TRUE),apply(data.y[,numerical,drop=FALSE],2,min0,na.rm=TRUE)),2,min,na.rm=TRUE)
+    rmax <- apply(rbind(apply(data.x[,numerical,drop=FALSE],2,max,na.rm=TRUE),apply(data.y[,numerical,drop=FALSE],2,max1,na.rm=TRUE)),2,max,na.rm=TRUE)
     r <- rmax-rmin
     r[r==0] <- 1
     for(i in seq_along(numerical)){
@@ -28,11 +42,16 @@ gowerD <- function(data.x, data.y = data.x, weights=NULL,numerical,factors,order
     data.y <- rbind(data.y,data.y)
     justone <- TRUE
   }
-  data.x <- apply(data.x,2,maxplus1)
-  data.y <- apply(data.y,2,maxplus1)
+  # Maximum + 1 for missing values
+  # TODO: find better way to handle missing in distance variables
+  maxplus1X <- apply(rbind(data.x,data.y),2,maxplus1)
+  for(i in 1:ncol(data.x)){
+    data.x[is.na(data.x[,i]),i] <- maxplus1X[i]
+    data.y[is.na(data.y[,i]),i] <- maxplus1X[i]
+  }
   levOrders <- as.numeric(levOrders)
   
-  out <- .Call( "gowerD", data.x, data.y,weights[weightind],c(length(numerical),length(factors),length(orders),length(mixed)),levOrders,mixed.constant,PACKAGE="VIM")
+  out <- .Call( "gowerD", as.matrix(data.x), as.matrix(data.y),weights[weightind],c(length(numerical),length(factors),length(orders),length(mixed)),levOrders,mixed.constant,PACKAGE="VIM")
   if(justone)
     out <-  out$delta[,1,drop=FALSE]
   else
